@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import os
 import sys
 
@@ -9,14 +23,17 @@ import time
 startup_debug = os.environ.get('NEW_RELIC_STARTUP_DEBUG',
         'off').lower() in ('on', 'true', '1')
 
-def log_message(text, *args):
-    if startup_debug:
+
+def log_message(text, *args, **kwargs):
+    critical = kwargs.get('critical', False)
+    if startup_debug or critical:
         text = text % args
         timestamp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
         sys.stdout.write('NEWRELIC: %s (%d) - %s\n' % (timestamp,
                 os.getpid(), text))
         sys.stdout.flush()
 
+
 log_message('New Relic Bootstrap (%s)', __file__)
 
 log_message('working_directory = %r', os.getcwd())
@@ -38,6 +55,8 @@ log_message('sys.path = %r', sys.path)
 
 for name in sorted(os.environ.keys()):
     if name.startswith('NEW_RELIC_') or name.startswith('PYTHON'):
+        if name == 'NEW_RELIC_LICENSE_KEY':
+            continue
         log_message('%s = %r', name, os.environ.get(name))
 
 # We need to import the original sitecustomize.py file if it exists. We
@@ -117,18 +136,27 @@ if python_prefix_matches and python_version_matches:
         # If it is a buildout created script, it will replace the whole
         # sys.path again later anyway.
 
-        if root_directory not in sys.path:
+        do_insert_path = root_directory not in sys.path
+        if do_insert_path:
             sys.path.insert(0, root_directory)
 
-        import newrelic.agent
+        import newrelic.config
 
         log_message('agent_version = %r', newrelic.version)
 
-        try:
-            del sys.path[sys.path.index(root_directory)]
-        except Exception:
-            pass
+        if do_insert_path:
+            try:
+                del sys.path[sys.path.index(root_directory)]
+            except Exception:
+                pass
 
         # Finally initialize the agent.
 
-        newrelic.agent.initialize(config_file, environment)
+        newrelic.config.initialize(config_file, environment)
+else:
+    log_message("""New Relic could not start because the newrelic-admin script was called from a Python installation that is different from the Python installation that is currently running. To fix this problem, call the newrelic-admin script from the Python installation that is currently running (details below).
+
+newrelic-admin Python directory: %r
+current Python directory: %r
+newrelic-admin Python version: %r
+current Python version: %r""", expected_python_prefix, actual_python_prefix, expected_python_version, actual_python_version, critical=True)
