@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """This module implements a generic object wrapper for use in performing
 monkey patching, helper functions to perform monkey patching and general
 purpose decorators and wrapper functions for various basic tasks one can
@@ -7,15 +21,14 @@ make use of when doing monkey patching.
 
 import sys
 import inspect
-import functools
 
-from ..packages import six
+from newrelic.packages import six
 
-from ..packages.wrapt import (ObjectProxy as _ObjectProxy,
+from newrelic.packages.wrapt import (ObjectProxy as _ObjectProxy,
         FunctionWrapper as _FunctionWrapper,
         BoundFunctionWrapper as _BoundFunctionWrapper)
 
-from ..packages.wrapt.wrappers import _FunctionWrapperBase
+from newrelic.packages.wrapt.wrappers import _FunctionWrapperBase
 
 # We previously had our own pure Python implementation of the generic
 # object wrapper but we now defer to using the wrapt module as its C
@@ -33,6 +46,7 @@ from ..packages.wrapt.wrappers import _FunctionWrapperBase
 # the intention eventually is that ObjectWrapper is deprecated. Either
 # ObjectProxy or FunctionWrapper should be used going forward.
 
+
 class _ObjectWrapperBase(object):
 
     def __setattr__(self, name, value):
@@ -81,12 +95,15 @@ class _ObjectWrapperBase(object):
     def _nr_parent(self):
         return self._self_parent
 
+
 class _NRBoundFunctionWrapper(_ObjectWrapperBase, _BoundFunctionWrapper):
     pass
 
+
 class FunctionWrapper(_ObjectWrapperBase, _FunctionWrapper):
     __bound_function_wrapper__ = _NRBoundFunctionWrapper
 
+
 class ObjectProxy(_ObjectProxy):
 
     def __setattr__(self, name, value):
@@ -123,6 +140,7 @@ class ObjectProxy(_ObjectProxy):
                     '_nr_last_object', self.__wrapped__)
             return self._self_last_object
 
+
 class CallableObjectProxy(ObjectProxy):
 
     def __call__(self, *args, **kwargs):
@@ -132,6 +150,7 @@ class CallableObjectProxy(ObjectProxy):
 # own code no longer uses it. It reaches down into what are wrapt internals
 # at present which shouldn't be doing.
 
+
 class ObjectWrapper(_ObjectWrapperBase, _FunctionWrapperBase):
     __bound_function_wrapper__ = _NRBoundFunctionWrapper
 
@@ -146,12 +165,10 @@ class ObjectWrapper(_ObjectWrapperBase, _FunctionWrapperBase):
         super(ObjectWrapper, self).__init__(wrapped, instance, wrapper,
                 binding=binding)
 
-# The wrap_callable() alias needs to be deprecated and usage of it removed.
-
-wrap_callable = FunctionWrapper
 
 # Helper functions for performing monkey patching.
 
+
 def resolve_path(module, name):
     if isinstance(module, six.string_types):
         __import__(module)
@@ -172,7 +189,7 @@ def resolve_path(module, name):
         # to work. For the case of a class we therefore access
         # the __dict__ directly. To cope though with the wrong
         # class being given to us, or a method being moved into
-        # a base class, we need to walk the class heirarchy to
+        # a base class, we need to walk the class hierarchy to
         # work out exactly which __dict__ the method was defined
         # in, as accessing it from __dict__ will fail if it was
         # not actually on the class given. Fallback to using
@@ -181,8 +198,8 @@ def resolve_path(module, name):
 
         if inspect.isclass(original):
             for cls in inspect.getmro(original):
-                if attribute in vars(original):
-                    original = vars(original)[attribute]
+                if attribute in vars(cls):
+                    original = vars(cls)[attribute]
                     break
             else:
                 original = getattr(original, attribute)
@@ -192,9 +209,11 @@ def resolve_path(module, name):
 
     return (parent, attribute, original)
 
+
 def apply_patch(parent, attribute, replacement):
     setattr(parent, attribute, replacement)
 
+
 def wrap_object(module, name, factory, args=(), kwargs={}):
     (parent, attribute, original) = resolve_path(module, name)
     wrapper = factory(original, *args, **kwargs)
@@ -207,6 +226,7 @@ def wrap_object(module, name, factory, args=(), kwargs={}):
 # instance attribute. Note that this cannot be used on attributes which
 # are themselves defined by a property object.
 
+
 class AttributeWrapper(object):
 
     def __init__(self, attribute, factory, args, kwargs):
@@ -225,6 +245,7 @@ class AttributeWrapper(object):
     def __delete__(self, instance):
         del instance.__dict__[self.attribute]
 
+
 def wrap_object_attribute(module, name, factory, args=(), kwargs={}):
     path, attribute = name.rsplit('.', 1)
     parent = resolve_path(module, path)[2]
@@ -235,6 +256,7 @@ def wrap_object_attribute(module, name, factory, args=(), kwargs={}):
 # Function for creating a decorator for applying to functions, as well as
 # short cut functions for applying wrapper functions via monkey patching.
 
+
 def function_wrapper(wrapper):
     def _wrapper(wrapped, instance, args, kwargs):
         target_wrapped = args[0]
@@ -247,14 +269,17 @@ def function_wrapper(wrapper):
         return FunctionWrapper(target_wrapped, target_wrapper)
     return FunctionWrapper(wrapper, _wrapper)
 
+
 def wrap_function_wrapper(module, name, wrapper):
     return wrap_object(module, name, FunctionWrapper, (wrapper,))
 
+
 def patch_function_wrapper(module, name):
     def _wrapper(wrapper):
         return wrap_object(module, name, FunctionWrapper, (wrapper,))
     return _wrapper
 
+
 def transient_function_wrapper(module, name):
     def _decorator(wrapper):
         def _wrapper(wrapped, instance, args, kwargs):
@@ -265,6 +290,7 @@ def transient_function_wrapper(module, name):
                 target_wrapper = wrapper.__get__(None, instance)
             else:
                 target_wrapper = wrapper.__get__(instance, type(instance))
+
             def _execute(wrapped, instance, args, kwargs):
                 (parent, attribute, original) = resolve_path(module, name)
                 replacement = FunctionWrapper(original, target_wrapper)
@@ -280,6 +306,7 @@ def transient_function_wrapper(module, name):
 # Generic decorators for performing actions before and after a wrapped
 # function is called, or modifying the inbound arguments or return value.
 
+
 def pre_function(function):
     @function_wrapper
     def _wrapper(wrapped, instance, args, kwargs):
@@ -290,12 +317,15 @@ def pre_function(function):
         return wrapped(*args, **kwargs)
     return _wrapper
 
+
 def PreFunctionWrapper(wrapped, function):
     return pre_function(function)(wrapped)
 
+
 def wrap_pre_function(module, object_path, function):
     return wrap_object(module, object_path, PreFunctionWrapper, (function,))
 
+
 def post_function(function):
     @function_wrapper
     def _wrapper(wrapped, instance, args, kwargs):
@@ -307,12 +337,15 @@ def post_function(function):
         return result
     return _wrapper
 
+
 def PostFunctionWrapper(wrapped, function):
     return post_function(function)(wrapped)
 
+
 def wrap_post_function(module, object_path, function):
     return wrap_object(module, object_path, PostFunctionWrapper, (function,))
 
+
 def in_function(function):
     @function_wrapper
     def _wrapper(wrapped, instance, args, kwargs):
@@ -336,20 +369,25 @@ def in_function(function):
 
     return _wrapper
 
+
 def InFunctionWrapper(wrapped, function):
     return in_function(function)(wrapped)
 
+
 def wrap_in_function(module, object_path, function):
     return wrap_object(module, object_path, InFunctionWrapper, (function,))
 
+
 def out_function(function):
     @function_wrapper
     def _wrapper(wrapped, instance, args, kwargs):
         return function(wrapped(*args, **kwargs))
     return _wrapper
 
+
 def OutFunctionWrapper(wrapped, function):
     return out_function(function)(wrapped)
 
+
 def wrap_out_function(module, object_path, function):
     return wrap_object(module, object_path, OutFunctionWrapper, (function,))
