@@ -23,6 +23,7 @@ the global defaults or those from local agent configuration.
 
 """
 
+from newrelic.common.object_names import parse_exc_info
 import os
 import logging
 import copy
@@ -109,6 +110,10 @@ class AttributesSettings(Settings):
     pass
 
 
+class GCRuntimeMetricsSettings(Settings):
+    pass
+
+
 class ThreadProfilerSettings(Settings):
     pass
 
@@ -320,6 +325,7 @@ class EventHarvestConfigHarvestLimitSettings(Settings):
 
 _settings = TopLevelSettings()
 _settings.attributes = AttributesSettings()
+_settings.gc_runtime_metrics = GCRuntimeMetricsSettings()
 _settings.thread_profiler = ThreadProfilerSettings()
 _settings.transaction_tracer = TransactionTracerSettings()
 _settings.transaction_tracer.attributes = TransactionTracerAttributesSettings()
@@ -445,7 +451,7 @@ def _environ_as_mapping(name, default=''):
     return result
 
 
-def _parse_ignore_status_codes(value, target):
+def _parse_status_codes(value, target):
     items = value.split()
     for item in items:
         try:
@@ -608,6 +614,9 @@ _settings.attributes.include = []
 _settings.thread_profiler.enabled = True
 _settings.cross_application_tracer.enabled = True
 
+_settings.gc_runtime_metrics.enabled = False
+_settings.gc_runtime_metrics.top_object_count_limit = 5
+
 _settings.transaction_events.enabled = True
 _settings.transaction_events.attributes.enabled = True
 _settings.transaction_events.attributes.exclude = []
@@ -644,9 +653,11 @@ _settings.transaction_tracer.attributes.include = []
 _settings.error_collector.enabled = True
 _settings.error_collector.capture_events = True
 _settings.error_collector.capture_source = False
-_settings.error_collector.ignore_errors = []
-_settings.error_collector.ignore_status_codes = _parse_ignore_status_codes(
+_settings.error_collector.ignore_classes = []
+_settings.error_collector.ignore_status_codes = _parse_status_codes(
         '100-102 200-208 226 300-308 404', set())
+_settings.error_collector.expected_classes = []
+_settings.error_collector.expected_status_codes = set()
 _settings.error_collector.attributes.enabled = True
 _settings.error_collector.attributes.exclude = []
 _settings.error_collector.attributes.include = []
@@ -997,6 +1008,12 @@ def apply_server_side_settings(server_side_config={}, settings=_settings):
         if value == 'apdex_f':
             agent_config['transaction_tracer.transaction_threshold'] = None
 
+    # If ignore_errors exists, and either ignore_classes is not set or it is empty
+    if "error_collector.ignore_errors" in agent_config and \
+        ("error_collector.ignore_classes" not in agent_config or not agent_config["error_collector.ignore_classes"]):
+        # Remap to newer config key
+        agent_config["error_collector.ignore_classes"] = agent_config.pop("error_collector.ignore_errors")
+
     # Overlay with agent server side configuration settings.
 
     for (name, value) in agent_config.items():
@@ -1072,4 +1089,83 @@ def _remove_ignored_configs(server_settings):
 
 
 def ignore_status_code(status):
+    """Legacy function kept here for compatibility."""
     return status in _settings.error_collector.ignore_status_codes
+
+
+def is_expected_error(
+    exc_info,
+    status_code=None,
+):
+    return error_matches_rules(
+        "expected",
+        exc_info,
+        status_code=status_code,
+    )
+
+
+def should_ignore_error(
+    exc_info,
+    status_code=None,
+):
+    return error_matches_rules(
+        "ignore",
+        exc_info,
+        status_code=status_code,
+    )
+
+
+def error_matches_rules(
+    rules_prefix,
+    exc_info,
+    status_code=None,
+):
+    # Delay imports to prevent lockups
+    from newrelic.api.application import application_instance
+    from newrelic.core.trace_cache import trace_cache
+
+    trace = trace_cache().current_trace()
+    settings = trace and trace.settings
+
+    if not settings:
+        # Retrieve application settings
+        application = application_instance()
+        settings = application and application.settings
+
+    # Default to global settings
+    settings = settings or global_settings()  
+
+    if not settings:
+        return False
+
+    # Retrieve settings based on prefix
+    classes_rules = getattr(settings.error_collector, "%s_classes" % rules_prefix, set())
+    status_codes_rules = getattr(settings.error_collector, "%s_status_codes" % rules_prefix, set())
+
+    module, name, fullnames, message = parse_exc_info(exc_info)
+    fullname = fullnames[0]
+
+    # Check class names
+    for fullname in fullnames:
+        if fullname in classes_rules:
+            return True
+
+    # Check status_code
+    # For callables, call on exc_info to retrieve status_code.
+    # It's possible to return None, in which case no code is evaluated.
+    if callable(status_code):
+        status_code = status_code(*exc_info)
+    
+    # Match status_code if it exists
+    if status_code is not None:
+        try:
+            # Coerce into integer
+            status_code = int(status_code)
+        except:
+            _logger.error("Failed to coerce status code into integer. "
+                          "status_code: %s" % str(status_code))
+        else:
+            if status_code in status_codes_rules:
+                return True
+
+    return False
