@@ -15,5 +15,5 @@
 from collections import namedtuple
 
 ErrorNode = namedtuple('ErrorNode',
-        ['timestamp', 'type', 'message', 'span_id', 'stack_trace', 'custom_params',
+        ['timestamp', 'type', 'message', 'expected', 'span_id', 'stack_trace', 'custom_params',
         'file_name', 'line_number', 'source'])
