@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import os
 import logging
 import time
@@ -11,11 +25,9 @@ import newrelic.packages.six as six
 import newrelic
 
 from newrelic.core.config import global_settings
-from newrelic.core.transaction_cache import transaction_cache
+from newrelic.core.trace_cache import trace_cache
 
-from newrelic.core.internal_metrics import (internal_trace, internal_metric)
-
-from ..common.encoding_utils import json_encode
+from newrelic.common.encoding_utils import json_encode
 
 try:
     from sys import intern
@@ -26,13 +38,11 @@ _logger = logging.getLogger(__name__)
 
 AGENT_PACKAGE_DIRECTORY = os.path.dirname(newrelic.__file__) + '/'
 
+
 class SessionState(object):
     RUNNING = 1
     FINISHED = 2
 
-class SessionType(object):
-    GENERIC = 1
-    XRAY = 2
 
 def format_stack_trace(frame, thread_category):
     """Formats the frame obj into a list of stack trace tuples.
@@ -87,18 +97,13 @@ def format_stack_trace(frame, thread_category):
     return stack_trace
 
 
-def collect_stack_traces(include_nr_threads=False, include_xrays=False):
+def collect_stack_traces(include_nr_threads=False):
     """Generator that yields the (thread category, stack trace) of all the
     python threads.
 
     """
     for (txn, thread_id, thread_category, frame) in \
-            transaction_cache().active_threads():
-
-        # Skip background threads for X-rays.
-
-        if (include_xrays) and (txn and txn.background_task):
-            continue
+            trace_cache().active_threads():
 
         # Skip NR Threads unless explicitly requested.
 
@@ -115,11 +120,9 @@ def collect_stack_traces(include_nr_threads=False, include_xrays=False):
         if not stack_trace:
             continue
 
-        if include_xrays and txn:
-            txn.add_profile_sample(stack_trace)
-
         yield thread_category, stack_trace
 
+
 class ProfileSessionManager(object):
     """Singleton class that manages multiple profile sessions. Do NOT
     instantiate directly from this class. Instead use profile_session_manager()
@@ -143,61 +146,19 @@ class ProfileSessionManager(object):
 
         self.full_profile_app = None
 
-        # Dict with app_name as key and another dictionary (let's call it child
-        # dictionary) as value. The child dictionary has key_txn name as key
-        # and xray profile session as value.
-
-        self.application_xrays = {}
         self.finished_sessions = defaultdict(list)
 
         self._profiler_shutdown = threading.Event()
         self._profiler_thread = None
         self._profiler_thread_running = False
         self._lock = threading.Lock()
-        self._xray_suspended = False
         self.profile_agent_code = False
         self.sample_period_s = 0.1
-        self._aggregation_time = 0.0
-
-    @internal_trace('Supportability/Profiling/Calls/add_stack_traces[sec|call]')
-    def add_stack_traces(self, app_name, key_txn, txn_type, stack_traces):
-        """Adds a list of stack_traces to a praticular xray profile session's
-        call tree. This is called at the end of a transaction when it's name is
-        frozen and it is identified as an xray transaction.
-
-        """
-
-        try:
-            xray_profile_sessions = self.application_xrays[app_name]
-            xps = xray_profile_sessions[key_txn]
-        except KeyError:
-            # Invalid xray_session name
-            return False
-
-        count = 0
-
-        with self._lock:
-            start = time.time()
-
-            for stack_trace in stack_traces:
-                count += len(stack_trace)
-                xps.update_call_tree(txn_type, stack_trace)
-                xps.sample_count += 1
-
-            end = time.time() - start
-            self._aggregation_time += end
-
-        internal_metric('Supportability/Profiling/Counts/stack_frames[frame]',
-                count)
-
-        return True
 
     def start_profile_session(self, app_name, profile_id, stop_time,
-            sample_period_s=0.1, profile_agent_code=False, key_txn=None,
-            xray_id=None):
-        """Start a new profiler session. If no key_txn is specified then start
-        a full_profiler.  If a full_profiler is already running, do nothing and
-        return false.
+            sample_period_s=0.1, profile_agent_code=False):
+        """Start a new profiler session. If a full_profiler is already
+        running, do nothing and return false.
 
         """
 
@@ -205,47 +166,22 @@ class ProfileSessionManager(object):
         # thread so do not need thread locking. If we move to multiple harvest
         # threads, would need locking.
 
-        profiler_type = SessionType.XRAY if key_txn else SessionType.GENERIC
-
         # Only one full profile session can run at any given time.
 
-        if profiler_type == SessionType.GENERIC and self.full_profile_session:
+        if self.full_profile_session:
             # log an error message
             return False
 
-        # Acquire thread lock before updating the datastructures. This method
-        # is invoked from the harvest thread and this ensures the varaibles are
+        # Acquire thread lock before updating the data structures. This method
+        # is invoked from the harvest thread and this ensures the variables are
         # not being updated concurrently by the profiler thread.
 
         with self._lock:
 
-            xray_profile_sessions = self.application_xrays.setdefault(app_name,
-                    {})
-
-            # Xray profiler session is already running for the key txn
-
-            if xray_profile_sessions.get(key_txn):
-                # XXX Maybe we should return True here since we are running
-                # one already. Currently doesn't care what response is, but
-                # if it might in future better to return True.
-
-                return False
-
             self.profile_agent_code = profile_agent_code
             self.sample_period_s = sample_period_s
-            ps = ProfileSession(profile_id, stop_time, xray_id, key_txn)
-            if key_txn is not None:
-                xray_profile_sessions[key_txn] = ps
-            else:
-                self.full_profile_session = ps
-                self.full_profile_app = app_name
-                self._xray_suspended = True
-
-                # If X-ray sessions are in progress then log the suspending of
-                # x-ray profiler
-
-                if xray_profile_sessions:
-                    _logger.debug('Suspending X-ray profiler.')
+            self.full_profile_session = ProfileSession(profile_id, stop_time)
+            self.full_profile_app = app_name
 
             # Create a background thread to collect stack traces. Do this only
             # if a background thread doesn't already exist.
@@ -260,7 +196,7 @@ class ProfileSessionManager(object):
 
         return True
 
-    def stop_profile_session(self, app_name, key_txn=None):
+    def stop_profile_session(self, app_name):
         """Stop a profiler session and return True when successful. Set key_txn
         to None to stop the full_profile_session. Returns False if no profiler
         sessions were stopped.
@@ -268,64 +204,36 @@ class ProfileSessionManager(object):
         """
 
         # Grab the lock before updating the profile sessions. This is to
-        # make sure that we're not updating the datastructures while the
+        # make sure that we're not updating the data structures while the
         # harvest thread is starting/stopping new sessions.
 
         with self._lock:
-            if key_txn is None:
-                if ((self.full_profile_session is not None) and (app_name ==
-                        self.full_profile_app)):
-                    self.full_profile_session.state = SessionState.FINISHED
-                    self.full_profile_session.actual_stop_time_s = time.time()
-                    self.finished_sessions[app_name].append(
-                            self.full_profile_session)
-                    self.full_profile_session = None
-                    self.full_profile_app = None
-                    self._xray_suspended = False
+            if ((self.full_profile_session is not None) and (app_name ==
+                    self.full_profile_app)):
+                self.full_profile_session.state = SessionState.FINISHED
+                self.full_profile_session.actual_stop_time_s = time.time()
+                self.finished_sessions[app_name].append(
+                        self.full_profile_session)
+                self.full_profile_session = None
+                self.full_profile_app = None
 
-                    # If X-ray sessions are pending then log the reenabling of
-                    # x-ray profiler
-
-                    if self.application_xrays[app_name]:
-                        _logger.debug('Resuming X-ray profiler.')
-                    return True
-                else:
-                    # log an error message
-                    return False
-
-            try:
-                xray_profile_sessions = self.application_xrays[app_name]
-                xps = xray_profile_sessions.pop(key_txn)
-                xps.state = SessionState.FINISHED
-                xps.actual_stop_time_s = time.time()
-                self.finished_sessions[app_name].append(xps)
-                return True
-            except KeyError:
+            else:
                 # log an error message
                 return False
 
     def profile_data(self, app_name):
         """Generator that yields the profile data in the form of call tree.
         Data from full profiler will only be returned when the profiler has
-        finished. Partial data from xray profiler sessions can be returned as
-        they become available. When no data is available None will be returned.
+        finished. When no data is available None will be returned.
 
         """
         with self._lock:
             for session in self.finished_sessions[app_name]:
-                if session.xray_id is not None:
-                    _logger.debug('Reporting final thread profiling data for '
-                            '%d transactions with name %r and xray ID of %r '
-                            'over a period of %.2f seconds and %d samples.',
-                            session.transaction_count, session.key_txn,
-                            session.xray_id, time.time()-session.start_time_s,
-                            session.sample_count)
-                else:
-                    _logger.debug('Reporting final thread profiling data for '
-                            '%d transactions over a period of %.2f seconds '
-                            'and %d samples.', session.transaction_count,
-                            time.time() - session.start_time_s,
-                            session.sample_count)
+                _logger.debug('Reporting final thread profiling data for '
+                        '%d transactions over a period of %.2f seconds '
+                        'and %d samples.', session.transaction_count,
+                        time.time() - session.start_time_s,
+                        session.sample_count)
 
                 yield session.profile_data()
 
@@ -334,45 +242,20 @@ class ProfileSessionManager(object):
 
             self.finished_sessions.pop(app_name)
 
-            xray_profile_sessions = self.application_xrays.get(app_name)
-            if xray_profile_sessions:
-                for xps in xray_profile_sessions.values():
-                    _logger.debug('Returning partial thread profiling data '
-                            'for %d transactions with name %r and xray ID of '
-                            '%r over a period of %.2f seconds and %d samples.',
-                            xps.transaction_count, xps.key_txn, xps.xray_id,
-                            time.time() - xps.start_time_s, xps.sample_count)
-
-                    yield xps.profile_data()
-
     def _profiler_loop(self):
         """Infinite loop that wakes up periodically to collect stack traces,
-        merge it into call tree if necessaray, finally update the state of all
+        merge it into call tree if necessary, finally update the state of all
         the active profile sessions.
 
         """
 
-        settings = global_settings()
-
-        overhead_threshold = settings.agent_limits.xray_profile_overhead
-
         while True:
 
-            # If xray profilers are not suspended and at least one x-ray
-            # session is active it'll cause collect_stack_traces() to add
-            # the stack_traces to the txn obj.
-
-            start = time.time()
-
-            include_xrays = ((not self._xray_suspended) and
-                    any(six.itervalues(self.application_xrays)))
-
             for category, stack in collect_stack_traces(
-                    self.profile_agent_code, include_xrays):
+                    self.profile_agent_code):
 
                 # Merge the stack_trace to the call tree only for
-                # full_profile_session. X-ray profiles will be merged at
-                # the time of exiting the transaction.
+                # full_profile_session.
 
                 if self.full_profile_session:
                     self.full_profile_session.update_call_tree(category,
@@ -382,30 +265,11 @@ class ProfileSessionManager(object):
 
             # Stop the profiler thread if there are no profile sessions.
 
-            if ((self.full_profile_session is None) and
-                    (not any(six.itervalues(self.application_xrays)))):
+            if self.full_profile_session is None:
                 self._profiler_thread_running = False
                 return
 
-            # Adjust sample period dynamically base on overheads of doing
-            # thread profiling if is an X Ray session.
-
-            if not self._xray_suspended:
-                overhead = time.time() - start
-
-                with self._lock:
-                    aggregation_time = self._aggregation_time
-                    self._aggregation_time = 0.0
-
-                overhead += aggregation_time
-
-                delay = overhead / self.sample_period_s / overhead_threshold
-                delay = min((max(1.0, delay) * self.sample_period_s), 5.0)
-
-                self._profiler_shutdown.wait(delay)
-
-            else:
-                self._profiler_shutdown.wait(self.sample_period_s)
+            self._profiler_shutdown.wait(self.sample_period_s)
 
     def update_profile_sessions(self):
         """Check the current time and decide if any of the profile sessions
@@ -419,26 +283,6 @@ class ProfileSessionManager(object):
                 self.stop_profile_session(self.full_profile_app)
                 _logger.info('Finished thread profiling session.')
 
-        if self._xray_suspended:
-            return
-
-        # Clean out the app_name entries with empty values
-
-        for app_name, xray_profile_sessions in \
-                list(six.iteritems(self.application_xrays)):
-            if not xray_profile_sessions:
-                self.application_xrays.pop(app_name)
-
-        # Update the xray_profile_sessions for each each application
-
-        for app_name, xray_profile_sessions in \
-                list(six.iteritems(self.application_xrays)):
-            for key_txn, xps in list(six.iteritems(xray_profile_sessions)):
-                if time.time() >= xps.stop_time_s:
-                    self.stop_profile_session(app_name, key_txn)
-                    _logger.info('Finished x-ray profiling session for %s',
-                            key_txn)
-
     def shutdown(self, app_name):
         """Stop all profile sessions running on the given app_name.
 
@@ -449,31 +293,15 @@ class ProfileSessionManager(object):
         if app_name == self.full_profile_app:
             self.stop_profile_session(app_name)
 
-        # Stop all xray profiler sessions.
-
-        try:
-            xray_profile_sessions = self.application_xrays[app_name]
-        except KeyError:
-            return False
-
-        for key_txn in list(six.iterkeys(xray_profile_sessions)):
-            self.stop_profile_session(app_name, key_txn)
-
         return True
 
 
 class ProfileSession(object):
-    def __init__(self, profile_id, stop_time, xray_id=None, key_txn=None):
+    def __init__(self, profile_id, stop_time):
         self.profile_id = profile_id
         self.start_time_s = time.time()
         self.stop_time_s = stop_time
         self.actual_stop_time_s = 0
-        self.xray_id = xray_id
-        self.key_txn = key_txn
-        if self.xray_id is None:
-            self.profiler_type = SessionType.GENERIC
-        else:
-            self.profiler_type = SessionType.XRAY
         self.state = SessionState.RUNNING
         self.reset_profile_data()
 
@@ -563,11 +391,8 @@ class ProfileSession(object):
 
         # Generic profiling sessions have to wait for completion before
         # reporting data.
-        #
-        # Xray profile session can send partial profile data on every harvest.
 
-        if ((self.profiler_type == SessionType.GENERIC) and
-                (self.state == SessionState.RUNNING)):
+        if self.state == SessionState.RUNNING:
             return None
 
         # We prune the number of nodes sent if we are over the specified
@@ -589,19 +414,8 @@ class ProfileSession(object):
                 flat_tree[category] = [x.flatten() for x in bucket.values()]
                 thread_count += len(bucket)
 
-        # If no profile data was captured for an x-ray session return None
-        # instead of sending an encoded empty data-structure. For a generic
-        # profiler continue to send an empty tree. This can happen on a system
-        # that uses green threads (coroutines), so sending an empty tree marks
-        # the end of a profile session. If we don't send anything then the UI
-        # timesout after a very long time (~15mins) which is frustrating for
-        # the customer.
-
-        if (thread_count == 0) and (self.profiler_type == SessionType.XRAY):
-            return None
-
         # Construct the actual final data for sending. The actual call
-        # data is turned into JSON, compessed and then base64 encoded at
+        # data is turned into JSON, compressed and then base64 encoded at
         # this point to cut its size.
 
         if settings.debug.log_thread_profile_payload:
@@ -609,24 +423,26 @@ class ProfileSession(object):
                     'payload=%r.', flat_tree)
 
         json_call_tree = json_encode(flat_tree)
+
+        level = settings.agent_limits.data_compression_level
+        level = level or zlib.Z_DEFAULT_COMPRESSION
+
         encoded_tree = base64.standard_b64encode(
-                zlib.compress(six.b(json_call_tree)))
+                zlib.compress(six.b(json_call_tree), level))
 
         if six.PY3:
             encoded_tree = encoded_tree.decode('Latin-1')
 
         profile = [[self.profile_id, self.start_time_s * 1000,
             (self.actual_stop_time_s or time.time()) * 1000, self.sample_count,
-            encoded_tree, thread_count, 0, self.xray_id]]
+            encoded_tree, thread_count, 0, None]]
 
-        # Reset the datastructures to default. For xray profile sessions we
-        # report the partial call tree at every harvest cycle. It is required
-        # to reset the datastructures to avoid aggregating the call trees
-        # across harvest cycles.
+        # Reset the data structures to default.
 
         self.reset_profile_data()
         return profile
 
+
 class CallTree(object):
     def __init__(self, method_data, call_count=0, depth=1):
         self.method_data = method_data
@@ -640,7 +456,7 @@ class CallTree(object):
         filename, func_name, func_line, exec_line = self.method_data
 
         # func_line is the first line of a function and exec_line is the line
-        # inside that fucntion that is currently being executed.  On the leaf
+        # inside that function that is currently being executed.  On the leaf
         # nodes the exec_line will be different from the func_line. Such nodes
         # are labeled with an @ sign in the second element of the tuple.
 
@@ -654,5 +470,6 @@ class CallTree(object):
         return [method_data, self.call_count, 0,
                 [x.flatten() for x in self.children.values() if not x.ignore]]
 
+
 def profile_session_manager():
     return ProfileSessionManager.singleton()
