@@ -41,7 +41,7 @@ from newrelic.common.encoding_utils import (unpack_field, json_encode,
 from newrelic.common.object_names import callable_name
 from newrelic.common.object_wrapper import (transient_function_wrapper,
         function_wrapper, wrap_function_wrapper, ObjectProxy)
-from newrelic.common.constants import LOCALHOST_EQUIVALENTS
+from newrelic.common.system_info import LOCALHOST_EQUIVALENTS
 
 from newrelic.config import initialize
 
@@ -491,6 +491,78 @@ def validate_transaction_metrics(name, group='Function',
     return _validate_wrapper
 
 
+def validate_time_metrics_outside_transaction(time_metrics=[], index=-1):
+    @function_wrapper
+    def _validate_wrapper(wrapped, instance, args, kwargs):
+
+        record_time_metric_called = []
+        recorded_metrics = []
+
+        @transient_function_wrapper('newrelic.core.stats_engine',
+                'StatsEngine.record_time_metric')
+        @catch_background_exceptions
+        def _validate_transaction_metrics(wrapped, instance, args, kwargs):
+            record_time_metric_called.append(True)
+            try:
+                result = wrapped(*args, **kwargs)
+            except:
+                raise
+            else:
+                metrics = instance.stats_table
+                # Record a copy of the metric value so that the values aren't
+                # merged in the future
+                _metrics = {}
+                for k, v in metrics.items():
+                    _metrics[k] = copy.copy(v)
+                recorded_metrics.append(_metrics)
+
+            return result
+
+        def _validate(metrics, name, count):
+            key = (name, '')
+            metric = metrics.get(key)
+
+            def _metrics_table():
+                out = ['']
+                out.append('Expected: {0}: {1}'.format(key, count))
+                for metric_key, metric_value in metrics.items():
+                    out.append('{0}: {1}'.format(metric_key, metric_value[0]))
+                return '\n'.join(out)
+
+            def _metric_details():
+                return 'metric=%r, count=%r' % (key, metric.call_count)
+
+            if count is not None:
+                assert metric is not None, _metrics_table()
+                if count == 'present':
+                    assert metric.call_count > 0, _metric_details()
+                else:
+                    assert metric.call_count == count, _metric_details()
+
+                assert metric.total_call_time >= 0, (key, metric)
+                assert metric.total_exclusive_call_time >= 0, (key, metric)
+                assert metric.min_call_time >= 0, (key, metric)
+                assert metric.sum_of_squares >= 0, (key, metric)
+
+            else:
+                assert metric is None, _metrics_table()
+
+        _new_wrapper = _validate_transaction_metrics(wrapped)
+        val = _new_wrapper(*args, **kwargs)
+        assert record_time_metric_called
+        metrics = recorded_metrics[index]
+
+        record_time_metric_called[:] = []
+        recorded_metrics[:] = []
+
+        for time_metric, count in time_metrics:
+            _validate(metrics, time_metric, count)
+
+        return val
+
+    return _validate_wrapper
+
+
 def capture_transaction_metrics(metrics_list, full_metrics=None):
     @transient_function_wrapper('newrelic.core.stats_engine',
             'StatsEngine.record_transaction')
@@ -845,6 +917,8 @@ def validate_non_transaction_error_event(required_intrinsics={}, num_errors=1,
                         'error.class'] == required_intrinsics['error.class']
                 assert intrinsics['error.message'].startswith(
                         required_intrinsics['error.message'])
+                assert intrinsics[
+                        'error.expected'] == required_intrinsics['error.expected']
                 now = time.time()
                 assert isinstance(intrinsics['timestamp'], int)
                 assert intrinsics['timestamp'] <= 1000.0 * now
@@ -1156,7 +1230,7 @@ def validate_transaction_trace_attributes(required_params={},
 
 
 def validate_transaction_error_trace_attributes(required_params={},
-        forgone_params={}):
+        forgone_params={}, exact_attrs={}):
     """Check the error trace for attributes, expect only one error to be
     present in the transaction.
     """
@@ -1176,7 +1250,7 @@ def validate_transaction_error_trace_attributes(required_params={},
             traced_error = error_data[0]
 
             check_error_attributes(traced_error.parameters, required_params,
-                    forgone_params, is_transaction=True)
+                    forgone_params, exact_attrs, is_transaction=True)
 
         return result
 
@@ -1184,7 +1258,7 @@ def validate_transaction_error_trace_attributes(required_params={},
 
 
 def check_error_attributes(parameters, required_params={}, forgone_params={},
-        is_transaction=True):
+        exact_attrs={}, is_transaction=True):
 
     parameter_fields = ['userAttributes']
     if is_transaction:
@@ -1200,10 +1274,10 @@ def check_error_attributes(parameters, required_params={}, forgone_params={},
     assert 'request_params' not in parameters
     assert 'request_uri' not in parameters
 
-    check_attributes(parameters, required_params, forgone_params)
+    check_attributes(parameters, required_params, forgone_params, exact_attrs)
 
 
-def check_attributes(parameters, required_params={}, forgone_params={}):
+def check_attributes(parameters, required_params={}, forgone_params={}, exact_attrs={}):
     if required_params:
         for param in required_params['agent']:
             assert param in parameters['agentAttributes']
@@ -1221,6 +1295,17 @@ def check_attributes(parameters, required_params={}, forgone_params={}):
         for param in forgone_params['user']:
             assert param not in parameters['userAttributes']
 
+    if exact_attrs:
+        for param, value in exact_attrs['agent'].items():
+            assert parameters['agentAttributes'][param] == value, (
+                    (param, value), parameters['agentAttributes'])
+        for param, value in exact_attrs['user'].items():
+            assert parameters['userAttributes'][param] == value, (
+                    (param, value), parameters['userAttributes'])
+        for param, value in exact_attrs['intrinsic'].items():
+            assert parameters['intrinsics'][param] == value, (
+                    (param, value), parameters['intrinsics'])
+
 
 def validate_error_trace_collector_json():
     @transient_function_wrapper('newrelic.core.stats_engine',
@@ -1603,9 +1688,9 @@ def validate_error_event_attributes(required_params={}, forgone_params={},
 
 
 def validate_error_trace_attributes_outside_transaction(err_name,
-        required_params={}, forgone_params={}):
+        required_params={}, forgone_params={}, exact_attrs={}):
     @transient_function_wrapper('newrelic.core.stats_engine',
-            'StatsEngine.record_exception')
+            'StatsEngine.notice_error')
     def _validate_error_trace_attributes_outside_transaction(wrapped, instance,
             args, kwargs):
         try:
@@ -1616,7 +1701,7 @@ def validate_error_trace_attributes_outside_transaction(err_name,
             target_error = core_application_stats_engine_error(err_name)
 
             check_error_attributes(target_error.parameters, required_params,
-                    forgone_params, is_transaction=False)
+                    forgone_params, exact_attrs, is_transaction=False)
 
         return result
 
@@ -1624,9 +1709,9 @@ def validate_error_trace_attributes_outside_transaction(err_name,
 
 
 def validate_error_event_attributes_outside_transaction(required_params={},
-        forgone_params={}):
+        forgone_params={}, exact_attrs=None, num_errors=None):
     @transient_function_wrapper('newrelic.core.stats_engine',
-            'StatsEngine.record_exception')
+            'StatsEngine.notice_error')
     def _validate_error_event_attributes_outside_transaction(wrapped, instance,
             args, kwargs):
 
@@ -1635,9 +1720,15 @@ def validate_error_event_attributes_outside_transaction(required_params={},
         except:
             raise
         else:
-            event_data = instance.error_events
+            event_data = [event for event in instance.error_events]
 
-            check_event_attributes(event_data, required_params, forgone_params)
+            if num_errors is not None:
+                exc_message = "Expected: %d, Got: %d. Verify StatsEngine is being reset before using this validator." % (num_errors, len(event_data))
+                assert num_errors == len(event_data), exc_message
+                    
+
+            for event in event_data:
+                check_event_attributes([event], required_params, forgone_params, exact_attrs=exact_attrs)
 
         return result
 
@@ -1834,6 +1925,7 @@ def validate_transaction_event_sample_data(required_attrs,
 
         assert 'error.class' not in intrinsics
         assert 'error.message' not in intrinsics
+        assert 'error.expected' not in intrinsics
         assert 'transactionName' not in intrinsics
 
         _validate_event_attributes(intrinsics,
@@ -1948,6 +2040,8 @@ def validate_error_event_sample_data(required_attrs={},
                         'error.class'] == required_attrs['error.class']
                 assert intrinsics['error.message'].startswith(
                         required_attrs['error.message'])
+                assert intrinsics[
+                        'error.expected'] == required_attrs['error.expected']
                 assert intrinsics['nr.transactionGuid'] is not None
                 assert intrinsics['spanId'] is not None
 
@@ -2066,7 +2160,7 @@ def validate_application_exception_message(expected_message):
     """Test exception message encoding/decoding for a single error"""
 
     @transient_function_wrapper('newrelic.core.stats_engine',
-            'StatsEngine.record_exception')
+            'StatsEngine.notice_error')
     def _validate_application_exception_message(wrapped, instance, args,
             kwargs):
 
@@ -2297,12 +2391,17 @@ def override_ignore_status_codes(status_codes):
     @function_wrapper
     def _override_ignore_status_codes(wrapped, instance, args, kwargs):
         try:
-            # Ignore_status_codes is only used directly
-            # from the global settings and not the application
-            # settings. We therefore need to patch the
-            # global settings directly.
+            # Updates can be made to ignored status codes in server
+            # side configs. Changes will be applied to application
+            # settings so we first check there and if they don't
+            # exist, we default to global settings
+
+            application = application_instance()
+            settings = application and application.settings
+
+            if not settings:
+                settings = global_settings()
 
-            settings = global_settings()
             original = settings.error_collector.ignore_status_codes
             settings.error_collector.ignore_status_codes = status_codes
             return wrapped(*args, **kwargs)
@@ -2371,7 +2470,7 @@ def core_application_stats_engine_error(error_type, app_name=None):
     and return the first error with the type of error_type. If none found,
     return None.
 
-    Useful for verifying that application.record_exception() works, since
+    Useful for verifying that application.notice_error() works, since
     the error is saved outside of a transaction. Must use a unique error
     type per test in a single test file, so that it returns the error you
     expect. (If you have 2 tests that record the same type of exception, then
@@ -2388,7 +2487,7 @@ def core_application_stats_engine_error(error_type, app_name=None):
 def error_is_saved(error, app_name=None):
     """Return True, if an error of a particular type has already been saved.
 
-    Before calling application.record_exception() in a test, it's good to
+    Before calling application.notice_error() in a test, it's good to
     check if that type of error has already been saved, so you know that
     there will only be a single example of a type of Error in error_data()
     when you verify that the exception was recorded correctly.
@@ -2400,7 +2499,7 @@ def error_is_saved(error, app_name=None):
             raise ErrorOne('error one message')
         except ErrorOne:
             application_instance = application()
-            application_instance.record_exception()
+            application_instance.notice_error()
 
         my_error = core_application_stats_engine_error(_error_one_name)
         assert my_error.message == 'error one message'
